from .autonicer import AutoNICER
from .autonicer import get_caldb_ver
from .autonicer import run
from .reprocess import Reprocess


__all__ = [
    "AutoNICER",
    "get_caldb_ver",
    "run",
    "Reprocess",
]



# AutoNICER
# Copyright 2022-2023 Nicholas Kuechel
# License Apache 2.0

import autonicer
import os
import logging
import shutil
import gzip
import tarfile
import pandas as pd
from astropy.io import fits
from termcolor import colored
import sys
import glob
import concurrent.futures

AUTONICER = os.path.basename(sys.argv[0])
logger = logging.getLogger(AUTONICER)


def extract_gz(file) -> str:
    with gzip.open(file, "rb") as gz_in:
        fname = str(file).split(".gz")
        with open(fname[0], "wb") as orig_out:
            shutil.copyfileobj(gz_in, orig_out)
    os.remove(file)
    return f"{file} -> {fname[0]}"


def extract_tar(file) -> str:
    with tarfile.open(file, "r:gz") as tfile:
        tfile.extractall()
    os.remove(file)
    return f"{file} extracted"


class Reprocess:
    def __init__(self, cals=None):
        if cals is None:
            self.curr_caldb = autonicer.get_caldb_ver()
        else:
            self.curr_caldb = cals
        self.base_dir = os.getcwd()
        self.last_caldb = None
        self.calstate = None
        self.src = False
        self.obsid = None
        self.ra = None
        self.dec = None
        self.bc_det = False
        self.comp_det = False
        self.reprocess_err = None
        self.clevts = self.get_clevts()

    def get_clevts(self):
        """
        Gets all cl.evt files associated with an existing NICER dataset
        """
        os.chdir("xti/event_cl/")
        files = glob.glob("*cl.evt")
        for i in files:
            self.get_meta(i)
        if f"bc{self.obsid}_0mpu7_cl.evt" in files:
            self.bc_det = True
        os.chdir(self.base_dir)
        return files

    def get_meta(self, infile):
        """
        Gets relevant metadata for reprocessing from NICER dataset
        """
        hdul = fits.open(infile)
        try:
            self.obsid = hdul[0].header["OBS_ID"]
        except KeyError:
            try:
                self.obsid = hdul[1].header["OBS_ID"]
            except KeyError:
                logger.info(colored("Unable to idenify OBSID", "red"))
                self.reprocess_err = True
        try:
            self.ra = hdul[0].header["RA_OBJ"]
            self.dec = hdul[0].header["DEC_OBJ"]

        except KeyError:
            logger.info(colored("Unable to identify required metadata.",
                                "red"))
            logger.info("Consider Re-downloading and reducing this dataset")
            logger.info("OR")
            logger.info("Try nicerl2 manually")
            self.reprocess_err = True
        return self.reprocess_err

    def checkcal(self):
        """
        Checks the status of an existing obsid's calibrations
        """
        logger.info(f"Latest NICER CALDB: {self.curr_caldb}")
        logger.info("")
        for i in self.clevts:
            os.chdir(f"{self.base_dir}/xti/event_cl/")
            hdul = fits.open(i)
            try:
                self.last_caldb = hdul[1].header["CALDBVER"]
                logger.info(f"CALDB for {i}: {self.last_caldb}")
            except KeyError:
                logger.info(colored("!!!!! CANNOT IDENTIFY CALDB !!!!!",
                                    "red"))
                break

            premessage = ""
            color = "red"
            if self.last_caldb == self.curr_caldb:
                self.calstate = True
                color = "green"
            else:
                self.calstate = False
                premessage = "NOT"
            logger.info(colored((f"{premessage} Up to date "
                                 "with latest NICER CALDB\n"),
                        color))
            hdul.close()
        os.chdir(self.base_dir)
        return self.calstate

    def decompress(self):
        """
        Extracts/decompresses all files in xti/event_cl
        in .gz or .tar.gz formats
        """
        logger.info(colored(f"######## Decompressing {self.obsid} ########",
                            "green"))
        os.chdir(f"{self.base_dir}/xti/event_cl/")
        gzs = glob.glob("*.evt.gz")
        with concurrent.futures.ThreadPoolExecutor() as executor:
            exts_gz = [executor.submit(extract_gz, i) for i in gzs]
            for j in concurrent.futures.as_completed(exts_gz):
                logger.info(j.result())

        tars = glob.glob("*.tar.gz")
        with concurrent.futures.ThreadPoolExecutor() as executor:
            ext_tar = [executor.submit(extract_tar, i) for i in tars]
            for j in concurrent.futures.as_completed(ext_tar):
                logger.info(j.result())
        if len(gzs) > 0 or len(tars) > 0:
            self.comp_det = True
        os.chdir(self.base_dir)
        return self.comp_det

    def reprocess(self, bc=None, compress=None):
        """
        Reprocesses an existing dataset with latest calibrations
        """
        if self.calstate is True:
            logger.info(f"Passing Reprocess of {self.obsid}\n")
        elif self.reprocess_err is True:
            logger.info(colored("!!!!! CANNOT REPROCESS !!!!!"), "red")
        else:
            self.decompress()
            if bc is True:
                self.bc_det = bc
            an = autonicer.AutoNICER(src=self.src, bc=self.bc_det, comp=False)
            reprocess_dict = {"OBSID": self.obsid,
                              "ra": self.ra,
                              "dec": self.dec}
            an.queue.append(reprocess_dict)
            proc_dir = self.base_dir.split(f"{self.obsid}")
            os.chdir(proc_dir[0])
            an.reduce(reprocess_dict)
            os.chdir(f"{self.base_dir}/xti/event_cl/")
            if compress is True or self.comp_det is True:
                an.nicer_compress()


def reprocess_check(argp, cals=None):
    """
    Parses and Runs --reprocess and --checkcal
    """
    check = Reprocess(cals)
    if argp.checkcal is True:
        check.checkcal()
    if argp.reprocess is True:
        check.reprocess(argp.bc, argp.compress)


def inlist(argp):
    """
    Runs --reprocess and/or checkcal for an input file
    with paths to NICER OBSID dirs or .evt files
    """
    cwd = os.getcwd()
    curr_cals = autonicer.get_caldb_ver()
    try:
        if len(argp.inlist) == 1:
            try:
                df = pd.read_csv(f"{argp.inlist[0]}")
                for i in df["Input"]:
                    path_sep = i.split("/xti/event_cl/")
                    os.chdir(path_sep[0])
                    if argp.checkcal is True or argp.reprocess is True:
                        reprocess_check(argp, curr_cals)
                    os.chdir(cwd)
            except IsADirectoryError:
                raise FileNotFoundError
        else:
            raise FileNotFoundError

    except FileNotFoundError:
        dirs = argp.inlist
        if len(argp.inlist) == 1:
            dirs = glob.glob(f"{argp.inlist[0]}")
        if len(dirs) != 0:
            for i in dirs:
                try:
                    os.chdir(i)
                    logger.info(f"Migrating to {colored(i, 'cyan')}")
                    if argp.checkcal is True or argp.reprocess is True:
                        reprocess_check(argp, curr_cals)
                    os.chdir(cwd)
                except NotADirectoryError:
                    logger.info(f"{i} is not a directory! Passing...")
        else:
            logger.info(colored("DATASETS NOT FOUND", "red"))
    except pd.errors.ParserError:
        logger.info(colored(f"Unable to resolve --inlist {argp.inlist[0]}",
                            "red"))
    except KeyError:
        logger.info(colored(f"{argp.inlist[0]} format not readable", "red"))
        logger.info("Format must be csv with Input column for inlist files.")



# AutoNICER
# Copyright 2022-2025 Nicholas Kuechel
# License Apache 2.0

import subprocess as sp
import os
import sys
import pandas as pd
import numpy as np
from astroquery.heasarc import Heasarc
from astropy.coordinates import SkyCoord
from astroquery import exceptions
from astropy.time import Time
from termcolor import colored
import datetime
import gzip
import shutil
import glob
import concurrent.futures
import logging
import argparse as ap
from .reprocess import reprocess_check
from .reprocess import inlist
from importlib.metadata import version
import asyncio
import aiohttp
from pathlib import Path
from tqdm import tqdm


AUTONICER = os.path.basename(sys.argv[0])
VERSION = version('autonicer')
logger = logging.getLogger(AUTONICER)


def get_caldb_ver():
    """
    Gets most up to nicer caldb version
    """
    caldb = sp.run("nicaldbver", shell=True,
                   capture_output=True, encoding="utf-8")
    convo = str(caldb.stdout).split("\n")
    return convo[0]


async def download_file(url: str) -> None:
    """
    Downloads data file from a specified url

    Parameters:
    url: str, url to fetch data from
    """
    split_url = url.split("/")
    file = '/'.join(split_url[7:])
    file = Path(file)
    file.parent.mkdir(exist_ok=True, parents=True)
    file_path = file.resolve()
    file_name = file.name
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as resp:
            if resp.status == 200:
                file_size = int(resp.headers['Content-Length'])
                with open(file_path, 'wb') as fd:
                    progress = tqdm(total=file_size, desc=file_name,
                                    unit="B", unit_scale=True, leave=False)
                    while True:
                        try:
                            chunk = await resp.content.read(1024)
                        except (asyncio.CancelledError,
                                aiohttp.ClientResponseError) as e:
                            logger.error(f"ERROR: Chunk read error: {e}\n")
                            break
                        if not chunk:
                            progress.close()
                            break
                        fd.write(chunk)
                        progress.update(len(chunk))
                    progress.close()
            else:
                logger.info(f"Download: {file_name} failed "
                            f"with code: {resp.status}\n")


async def download_obsid(urls: list) -> None:
    """
    Downloads all files from urls to make up an entire obsid dataset

    Parameters:
    urls: list, urls to all files that make up an entire obsid dataset
    """
    sem = asyncio.Semaphore(4)
    async with sem:
        tasks = []
        for url in urls:
            task = asyncio.create_task(download_file(url))
            tasks.append(task)
        await asyncio.gather(*tasks)


class AutoNICER(object):
    def __init__(self, src=None, bc=None, comp=None):
        self.st = True
        self.xti = 0
        self.queue = []
        self.caldb_ver = ""
        self.obj = src
        self.bc_sel = bc
        self.q_set = "n"
        self.tar_sel = comp
        self.q_path = 0
        self.q_name = 0
        self.startup()

    def startup(self):
        """
        Prompted setting for if autonicer is just called w/ no paramerters set
        """

        def null_parse(var):
            if var == "" or var is True:
                var = "y"
            else:
                var = "n"
            return var

        logq = False
        if self.obj is None and self.bc_sel is None and self.tar_sel is None:
            logq = True
        if self.obj is None:
            self.obj = str(input("Target: "))
        if self.bc_sel is None:
            self.bc_sel = str(input("Apply Bary-Center Correction: [y] "))
        if self.tar_sel is None:
            self.tar_sel = str(input("Compress XTI files (.gz): [y] "))
        if logq is True:
            self.q_set = str(input("Write Output Log: [n] "))
            self.q_set = self.q_set.lower()
            if self.q_set == "y":
                ne = str(input("New or Add to existing Log: "))
                if ne.lower() == "add":
                    self.q_path = str(input("Input Que: "))
                    if self.q_path[0] == r"'" or self.que_path[0] == r'"':
                        self.q_path = self.q_path.replace("'", "")
                        self.q_path = self.q_path.replace(" ", "")
                        self.q_path = self.q_path.replace('"', "")
                elif ne.lower() == "new":
                    self.q_name = str(input("Name of log file (no .csv): "))

                else:
                    self.q_set = "n"
        self.bc_sel = null_parse(self.bc_sel)
        self.tar_sel = null_parse(self.tar_sel)

    def call_nicer(self):
        """
        Querys the nicermastr catalog for all observations
        of the specified source(self.obj)
        """
        heasarc = Heasarc()
        try:
            Heasarc.clear_cache()
            position = SkyCoord.from_name(self.obj)
            xti = heasarc.query_region(
                position, catalog="nicermastr"
            )  # calls NICER master catalogue for an input object
        except exceptions.InvalidQueryError:
            logger.info(colored(f"UNABLE TO RESOLVE {self.obj} in HEASARC!",
                                "red"))
            logger.info(colored("Exiting ...", "red"))
            exit()
        else:
            xti = xti.to_pandas()
            xti.columns = [name.upper() for name in xti.columns]
            xti["TIME"] = Time(xti["TIME"], format="mjd").to_datetime()
            self.xti = xti

    def make_cycle(self):
        """
        Makes a Cycle# column in resulting query table from obsid table
        """
        cycle = []
        for i in self.xti["OBSID"]:
            convo = np.floor(float(i) * 10 ** (-9))
            cycle.append(int(convo))
        self.xti["Cycle#"] = cycle
        return self.xti

    def sel_obs(self, enter):
        """
        Selects and queues up the obsid desired
        """
        selstate = True
        queued_obsids = [i["OBSID"] for i in self.queue]
        if enter in queued_obsids:
            logger.info(f"{enter} is already queued up... ignoring")
        else:
            try:
                row = self.xti.loc[self.xti["OBSID"] == enter]
                dt = row["TIME"]
                row.reset_index(drop=True, inplace=True)
                dt.reset_index(drop=True, inplace=True)
                year = str(dt[0].year)
                month = dt[0].month
                # basic if else statement to fix single digit months
                # not having a zero out front
                if month < 10:
                    month = "0" + str(month)
                else:
                    month = str(month)
                sel_dict = {"OBSID": enter,
                            "month": month,
                            "year": year,
                            "ra": row["RA"][0],
                            "dec": row["DEC"][0]
                            }
                logger.info(f"Adding {enter}")
                self.queue.append(sel_dict)
                return selstate
            except KeyError:
                logger.info(colored("OBSID NOT FOUND!", "red"))
                selstate = False
                return selstate

    def rm_obs(self, cmd):
        """
        Manages the removal command types
        """
        if cmd.lower() == "all":
            self.queue.clear()
        else:
            n = 0
            if cmd.lower() == "back":
                n = -1
            else:
                sel_obsids = [i["OBSID"] for i in self.queue]
                n = sel_obsids.index(cmd)
            del self.queue[n]

    def commands(self, enter):
        if enter[0].lower() == "done":
            # Command to finish selection of obs.
            self.st = False
            self.pull_reduce()

        elif enter[0].lower() == "sel":
            # displays all selected observations in the cmd line
            logger.info("Observations Selected:")
            for i in self.queue:
                logger.info(i["OBSID"])
            return True

        elif enter[0] is None or enter[0] == "":
            # Error message for nothing entered in the prompt
            logger.info("Nothing entered...")
            logger.info("!!!ENTER SOMETHING!!!")
            return True

        elif enter[0].lower() == "back":
            # Deletes the previously entered obsid
            try:
                logger.info(f"Removing {self.queue[-1]['OBSID']}")
            except IndexError:
                logger.info(colored("Nothing found to Remove!", "red"))
            else:
                self.rm_obs("back")
            return True

        elif enter[0].lower() == "cycle":
            row = self.make_cycle().loc[
                self.make_cycle()["Cycle#"] == float(enter[1])
            ]
            for i in row["OBSID"]:
                self.sel_obs(i)
            return True

        elif enter[0].lower() == "rm":
            try:
                self.rm_obs(enter[1])
            except ValueError:
                logger.info(colored("Nothing found to Remove!", "red"))
            return True

        elif enter[0] == "settings":
            logger.info(f"Target: {self.obj}")
            logger.info(f"Barycenter Correction: {self.bc_sel}")
            if self.q_set == "y":
                logger.info(f"Log Name: {self.q_name}")
                logger.info(f"Output Log: {self.q_set}")
            logger.info(f".gz compresion: {self.tar_sel}")

        elif enter[0] == "exit":
            exit()

        else:
            try:
                if int(enter[0]) > (10**8):
                    obsst = self.sel_obs(enter[0])
                    return obsst
                else:
                    raise ValueError
            except ValueError:
                logger.info("Unknown Entry")
                return False

    def command_center(self, enter=None):
        """
        Function to manage all the commands
        """
        # prompts the user to select obs to be pulled and reduced
        orig_in = enter
        cmdstate = None
        self.st = True
        while self.st is True:
            if orig_in is not None:
                cmdstate = self.commands(enter.split(" "))
                self.st = False
            else:
                cmdstate = enter = str(
                    input(colored("autoNICER", "blue") + " > ")
                ).split(" ")
                self.commands(enter)
        return cmdstate

    def nicer_compress(self):
        """
        compresses .evt files
        """
        logger.info(colored("##########  .gz compression  ##########",
                    "green"))

        def gz_comp(file):
            """
            .gz compression of a single file and
            removal of original file after compression
            """
            if file == "":
                pass
            else:
                with open(file, "rb") as f_in:
                    with gzip.open(f"{file}.gz", "wb") as f_out:
                        shutil.copyfileobj(f_in, f_out)
                os.remove(file)
                return f"{file} -> {file}.gz"

        logger.info("\nCompressing ufa.evt files")
        logger.info("-" * 50)
        # files and loop to compress the ufa files
        files = glob.glob("*ufa.evt")
        with concurrent.futures.ThreadPoolExecutor() as executor:
            comps = [executor.submit(gz_comp, i) for i in files]

            for j in concurrent.futures.as_completed(comps):
                logger.info(j.result())

        # compression of the non-bc mpu7_cl.evt file
        # if barycenter correction is selected
        if self.bc_sel.lower() == "y":
            logger.info("\nCompressing cl.evt files")
            logger.info("-" * 50)
            cl_file = glob.glob("ni*cl.evt")
            for i in cl_file:
                logger.info(gz_comp(i))

    def add2q(self, q, base_dir, obsid):
        """
        Adds processed data to existing log
        """
        newline = pd.DataFrame(
            {
                "Input": [(f"{base_dir}/{obsid}/xti/"
                           f"event_cl/bc{obsid}_0mpu7_cl.evt")],
                "OBSID": [f"NI{obsid}"],
                "CALDB": [f"{self.caldb_ver}"],
                "DateTime": [f"{datetime.datetime.now()}"],
            },
        )
        q = pd.concat([q, newline])
        q.to_csv(self.q_path, index=False)

    def _make_download_links(self, info: dict) -> dict:
        """
        Makes all the download links for each file of an OBSID

        Parameters:
        info: dict, information from AutoNICER.queue
                    required to make all download links
                    (OSBID, year, month, ra, dec)

        Returns:
        list, all links as str for the given OBSID dataset
        """
        base_url = "https://nasa-heasarc.s3.amazonaws.com/nicer/data/obs/"
        base_url = f"{base_url}{info['year']}_{info['month']}/{info['OBSID']}"
        file_urls = {
            "small": [f"/log/ni{info['OBSID']}_errlog.html",
                      f"/log/ni{info['OBSID']}_joblog.html",
                      f"/auxil/ni{info['OBSID']}.orb.gz",
                      f"/auxil/ni{info['OBSID']}.cat",
                      f"/xti/hk/ni{info['OBSID']}_0mpu0.hk.gz",
                      f"/xti/hk/ni{info['OBSID']}_0mpu1.hk.gz",
                      f"/xti/hk/ni{info['OBSID']}_0mpu2.hk.gz",
                      f"/xti/hk/ni{info['OBSID']}_0mpu3.hk.gz",
                      f"/xti/hk/ni{info['OBSID']}_0mpu4.hk.gz",
                      f"/xti/hk/ni{info['OBSID']}_0mpu5.hk.gz",
                      f"/xti/hk/ni{info['OBSID']}_0mpu6.hk.gz",
                      f"/xti/products/ni{info['OBSID']}_lc.png",
                      f"/xti/products/ni{info['OBSID']}_pi.png",
                      f"/xti/products/ni{info['OBSID']}mpu7_load.xcm.gz",
                      f"/xti/products/ni{info['OBSID']}mpu7_sr.lc.gz",
                      f"/xti/products/ni{info['OBSID']}mpu7_sr.pha.gz",
                      f"/xti/products/ni{info['OBSID']}mpu7_bg.pha.gz"],
            "big": [f"/auxil/ni{info['OBSID']}.mkf.gz",
                    f"/auxil/ni{info['OBSID']}.att.gz",
                    f"/xti/event_cl/ni{info['OBSID']}_0mpu7_cl.evt.gz",
                    f"/xti/event_cl/ni{info['OBSID']}_0mpu7_ufa.evt.gz",
                    f"/xti/event_uf/ni{info['OBSID']}_0mpu0_uf.evt.gz",
                    f"/xti/event_uf/ni{info['OBSID']}_0mpu1_uf.evt.gz",
                    f"/xti/event_uf/ni{info['OBSID']}_0mpu2_uf.evt.gz",
                    f"/xti/event_uf/ni{info['OBSID']}_0mpu3_uf.evt.gz",
                    f"/xti/event_uf/ni{info['OBSID']}_0mpu4_uf.evt.gz",
                    f"/xti/event_uf/ni{info['OBSID']}_0mpu5_uf.evt.gz",
                    f"/xti/event_uf/ni{info['OBSID']}_0mpu6_uf.evt.gz",
                    f"/xti/products/ni{info['OBSID']}mpu7.arf.gz",
                    f"/xti/products/ni{info['OBSID']}mpu7.rmf.gz",
                    f"/xti/products/ni{info['OBSID']}mpu7_bg.pha.gz",
                    f"/xti/products/ni{info['OBSID']}mpu7_sk.arf.gz"]
        }
        return {"small": [f"{base_url}{url}" for url in file_urls["small"]],
                "big": [f"{base_url}{url}" for url in file_urls["big"]]}

    def reduce(self, data):
        """
        Performs standardized data reduction scheme calling
        nicerl2 and barycorr if set
        """
        logger.info("\nStarting Data Reduction... \n")
        sp.call(f"nicerl2 indir={data['OBSID']}/ clobber=yes", shell=True)
        if self.bc_sel.lower() == "y":
            sp.call(
                (f"barycorr infile={data['OBSID']}/xti/event_cl/"
                 f"ni{data['OBSID']}_0mpu7_cl.evt outfile={data['OBSID']}"
                 f"/xti/event_cl/bc{data['OBSID']}_0mpu7_cl.evt "
                 f"orbitfiles={data['OBSID']}/auxil/ni{data['OBSID']}.orb "
                 f"refframe=ICRS ra={data['ra']} "
                 f"dec={data['dec']} ephem=JPLEPH.430 clobber=yes"),
                shell=True,
            )

    def pull_reduce(self):
        """
        Downloads the NICER data
        Puts the retrieved data through a standardized data reduction scheme
        """

        for data in self.queue:
            logger.info("")
            logger.info("-" * 60)
            logger.info((" " * 14) + "Prosessing OBSID: " +
                        colored(str(data["OBSID"]), "cyan"))
            logger.info("-" * 60)
            # Download dataset
            urls = self._make_download_links(data)
            # Download big
            logger.info("\nDownloading large files\n")
            for url in urls["big"]:
                asyncio.run(download_file(url))
            # Download small
            logger.info("\nDownloading small files\n")
            asyncio.run(download_obsid(urls["small"]))

            self.caldb_ver = get_caldb_ver()
            self.reduce(data)

            base_dir = os.getcwd()
            os.chdir(f"{data['OBSID']}/xti/event_cl/")
            if self.tar_sel.lower() == "y":
                self.nicer_compress()
            os.chdir(base_dir)
            if self.q_set == "y" and self.q_path != 0:
                read_q = pd.read_csv(self.q_path)
                self.add2q(read_q, base_dir, data['OBSID'])
            elif self.q_set == "y" and self.q_path == 0:
                q = pd.DataFrame(
                    {"Input": [], "OBSID": [], "CALDB": [], "DateTime": []}
                )
                self.q_path = f"{base_dir}/{self.q_name}.csv"
                self.add2q(q, base_dir, data['OBSID'])
            else:
                pass


def run(args=None):
    p = ap.ArgumentParser(
        description=("A program for piplining NICER data reduction. "
                     "Run by just typing autonicer.")
    )

    p.add_argument(
        "-src",
        "--src",
        help="Set the src from the command line",
        type=str,
    )

    p.add_argument(
        "-checkcal",
        "--checkcal",
        help=("Checks if mpu7_cl.evt are up to date "
              "with latest NICER calibrations"),
        action="store_true",
        default=False,
    )

    p.add_argument(
        "-reprocess",
        "--reprocess",
        help="Engages a reprocessing of calibrations ",
        action="store_true",
        default=False,
    )

    p.add_argument(
        "-bc",
        "--bc",
        help=("Engages option for a barycenter correction "
              "in data reduction procedure"),
        action="store_true",
        default=None,
    )

    p.add_argument(
        "-compress",
        "--compress",
        help="Engages option for .gz compression of ufa.evt files",
        action="store_true",
        default=None,
    )

    p.add_argument(
        "-i",
        "--inlist",
        dest="inlist",
        help=(".csv or Unix style pathname pattern whhich lists paths "
              "to OBSID dirs or mpu7_cl.evt "
              "files for use with --reprocess and/or --checkcal"),
        default=None,
        nargs="+",
    )

    p.add_argument(
        "--version",
        action="version",
        version=f"%(prog)s {VERSION}"
    )

    argp = p.parse_args(args)

    level = logging.INFO
    logging.basicConfig(stream=sys.stdout,
                        level=level,
                        format="")

    if argp.checkcal is True or argp.reprocess is True:
        if argp.inlist is None:
            reprocess_check(argp)
        else:
            inlist(argp)
    else:
        logger.info(colored("##########  Auto NICER  ##########\n",
                    "cyan"))
        an = AutoNICER(argp.src, argp.bc, argp.compress)
        an.call_nicer()
        an.command_center()



import autonicer
from astropy.time import Time
from astropy.io import fits
import datetime
import numpy as np
import os
import pytest
import shutil
import subprocess as sp
import tarfile
import glob
from termcolor import colored

base_dir = os.getcwd()
os.mkdir("data")
os.chdir("data")
an = autonicer.AutoNICER(src="PSR_B0531+21", bc=True, comp=True)


def test_passin():
    assert an.obj == "PSR_B0531+21"
    assert an.bc_sel == "y"
    assert an.tar_sel == "y"


def test_call_nicer():
    an.call_nicer()
    for i in an.xti["OBSID"]:
        assert isinstance(i, str) == True
        assert isinstance(i, bytes) == False
    for i in an.xti["TIME"]:
        assert isinstance(i, float) == False
        assert isinstance(i, datetime.datetime) == True


def test_make_cycle():
    an.make_cycle()
    assert len(an.xti["Cycle#"]) == len(an.xti["OBSID"])
    cnt = 0
    for i in an.xti["OBSID"]:
        cyc = an.xti.loc[cnt, "Cycle#"]
        convo = float(i) * (10 ** (-9))
        assert np.floor(convo) == cyc
        cnt += 1


def test_selections():
    # test selecting 1 obsid
    an.command_center("1013010112")
    assert an.queue[0]['OBSID'] == "1013010112"
    assert len(an.queue) == 1
    # test short entry/ incomplete obsid
    an.command_center("11")
    assert len(an.queue) == 1
    # test obsid from a different source not queried
    an.command_center("1013010000")
    assert len(an.queue) == 1
    # test selecting by entire cycle
    an.command_center("cycle 1")
    assert len(an.queue) == 59
    # test removal of specific obsid
    an.command_center("rm 1013010112")
    sel_obsids = [i["OBSID"] for i in an.queue]
    assert "1013010112" not in sel_obsids
    assert len(an.queue) == 58
    # test removal of previously inserted obsid
    last_obsid = sel_obsids[-1]
    an.command_center("back")
    sel_obsids = [i["OBSID"] for i in an.queue]
    assert last_obsid not in sel_obsids
    assert len(an.queue) == 57
    # test duplicates not getting added to the queue
    an.command_center("cycle 1")
    assert len(an.queue) == 59
    an.command_center("1013010112")
    assert len(an.queue) == 59
    an.command_center("rm all")
    assert len(an.queue) == 0


def test_get_caldbver():
    assert autonicer.get_caldb_ver() == "xti20240206"


def test_pullreduce():
    an.q_set = "y"
    an.q_name = "test"
    an.command_center("3013010102")
    sel = an.command_center("sel")
    assert sel is True
    an.command_center("done")
    os.chdir(f"{base_dir}/data/3013010102/xti/event_cl/")
    ufa = glob.glob("*ufa.evt")
    assert len(ufa) == 0
    ufa_gz = glob.glob("*ufa.evt.gz")
    assert len(ufa_gz) == 8
    cl = glob.glob("*cl.evt")
    assert len(cl) == 1
    cl_gz = glob.glob("*cl.evt.gz")
    assert len(cl_gz) == 1
    os.chdir(f"{base_dir}")


@pytest.fixture
def setup_reprocess():
    os.chdir(f"{base_dir}/data/3013010102/")
    return autonicer.Reprocess()


def test_get_clevts(setup_reprocess):
    check = setup_reprocess
    assert len(check.clevts) == 1
    for i in check.clevts:
        assert i == "bc3013010102_0mpu7_cl.evt"
    assert check.bc_det is True
    os.chdir(f"{base_dir}/data/")


def test_getmeta(setup_reprocess):
    check = setup_reprocess
    assert check.obsid == "3013010102"
    assert check.reprocess_err == None
    os.chdir(f"{base_dir}/data/")


metadata = {
    "OBS_ID": None,
    "RA_OBJ": None,
    "DEC_OBJ": None,
}


def test_nometa():
    os.chdir(f"{base_dir}/data/3013010102/xti/event_cl/")
    cl_files = glob.glob("*cl.evt")
    for i in cl_files:
        hdul = fits.open(i)
        assert hdul[0].header["OBS_ID"] == hdul[1].header["OBS_ID"]
        for j, k in metadata.items():
            metadata[j] = hdul[0].header[j]
            del hdul[0].header[j]
        del hdul[1].header["OBS_ID"]
        hdul.writeto(i, overwrite=True)
        hdul.close()
    os.chdir(f"{base_dir}/data/3013010102")
    check = autonicer.Reprocess()
    check.reprocess()
    assert check.reprocess_err is True


def test_no_primary_obsid():
    os.chdir(f"{base_dir}/data/3013010102/xti/event_cl/")
    cl_files = glob.glob("*cl.evt")
    for i in cl_files:
        hdul = fits.open(i)
        hdul[1].header["OBS_ID"] = metadata["OBS_ID"]
        hdul[0].header["RA_OBJ"] = metadata["RA_OBJ"]
        hdul[0].header["DEC_OBJ"] = metadata["DEC_OBJ"]
        hdul.writeto(i, overwrite=True)
        hdul.close()
    os.chdir(f"{base_dir}/data/3013010102")
    check2 = autonicer.Reprocess()
    assert check2.reprocess_err is None
    assert check2.src is False


def test_decompress(setup_reprocess):
    os.chdir(f"{base_dir}/data/3013010102/xti/event_cl/")
    with open("dummy.txt", "w") as dummy:
        pass
    with tarfile.open("dummy.tar.gz", "w:gz") as tar:
        tar.add("dummy.txt")
    check = setup_reprocess
    comp_det = check.decompress()
    assert comp_det is True
    gzs = glob.glob("*evt.gz")
    tars = glob.glob("*.tar.gz")
    assert len(gzs) == 0
    assert len(tars) == 0
    os.chdir(f"{base_dir}/data/")


def test_checkcal(setup_reprocess):
    check = setup_reprocess
    check.checkcal()
    assert check.calstate == True
    os.chdir(f"{base_dir}/data/")


def test_no_caldb(setup_reprocess):
    os.chdir(f"{base_dir}/data/3013010102/xti/event_cl/")
    cl_files = glob.glob("*cl.evt")
    for i in cl_files:
        hdul = fits.open(i)
        del hdul[1].header["CALDBVER"]
        hdul.writeto(i, overwrite=True)
        hdul.close()
    check = setup_reprocess
    check.checkcal()


def get_processed_time(file):
    hdul = fits.open(file)
    dt_created = hdul[1].header["DATE"]
    hdul.close()
    return dt_created


def test_reprocess(setup_reprocess):
    check = setup_reprocess
    os.chdir(f"{check.base_dir}/xti/event_cl/")
    pbc_dt_str = get_processed_time(f"bc{check.obsid}_0mpu7_cl.evt")
    pre_bc_dt = datetime.datetime.strptime(pbc_dt_str, "%Y-%m-%dT%H:%M:%S")
    os.chdir(f"{check.base_dir}")
    autonicer.run(["--reprocess", "--bc", "--compress"])
    os.chdir(f"{check.base_dir}/xti/event_cl/")
    pobc_dt_str = get_processed_time(f"bc{check.obsid}_0mpu7_cl.evt")
    post_bc_dt = datetime.datetime.strptime(pobc_dt_str, "%Y-%m-%dT%H:%M:%S")
    assert post_bc_dt > pre_bc_dt


def test_checkcal_reprocess():
    try:
        os.chdir(f"{base_dir}/data")
        files = ["test.csv", "fail.lis", "*"]
        for i in files:
            autonicer.run(["--checkcal",
                           "--reprocess",
                           "--bc",
                           "--compress",
                           f"--inlist={i}"])
    except SystemExit:
        pass


def test_inlist_singledir():
    os.chdir(f"{base_dir}/data")
    os.remove("test.csv")
    autonicer.run(["--checkcal", "--inlist", "*"])


def test_inlist_readin():
    os.chdir(f"{base_dir}")
    files = ["README.md", "fail.lis"]
    for i in files:
        autonicer.run(["--checkcal", "-i", f"{i}"])


def test_cleanup():
    os.chdir(base_dir)
    shutil.rmtree("data/")
